@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( Tickit::Widget );
 
-our $VERSION = '0.10';
+our $VERSION = '0.11';
 
 use Carp;
 
@@ -99,21 +99,20 @@ sub render
    my $self = shift;
    my %args = @_;
 
-   my $bottom = $args{top} + $args{lines} - 1;
-
-   my $window = $self->window or return;
+   my $win = $self->window or return;
+   my $rect = $args{rect};
 
    my ( $above ) =
-      Tickit::Utils::align( 1, $window->lines, $self->valign );
+      Tickit::Utils::align( 1, $win->lines, $self->valign );
 
-   my $cols = $window->cols;
+   my $cols = $win->cols;
 
-   $window->goto( $_, 0 ), $window->erasech( $cols ) for $args{top} .. $above - 1;
+   $win->goto( $_, 0 ), $win->erasech( $cols ) for $rect->top .. $above - 1;
 
-   $window->goto( $above, 0 );
+   $win->goto( $above, 0 );
    $self->render_line;
 
-   $window->goto( $_, 0 ), $window->erasech( $cols ) for $above + 1 .. $bottom;
+   $win->goto( $_, 0 ), $win->erasech( $cols ) for $above + 1 .. $rect->bottom - 1;
 }
 
 =head1 SUBCLASS METHODS
